@@ -6,14 +6,12 @@ import (
 	"fmt"
 	"io"
 	"log"
+	"log/slog"
 	"net/http"
 	"os"
 	"runtime"
-	"strconv"
-	"strings"
 	"time"
 
-	"github.com/adrg/xdg"
 	tea "github.com/charmbracelet/bubbletea"
 	"github.com/charmbracelet/lipgloss"
 	"github.com/pelletier/go-toml/v2"
@@ -24,7 +22,7 @@ import (
 
 // Run superfile app
 func Run(content embed.FS) {
-
+	internal.LoadInitial_PrerenderedVariables()
 	internal.LoadAllDefaultConfig(content)
 
 	app := &cli.App{
@@ -66,6 +64,12 @@ func Run(content embed.FS) {
 				Usage:   "Print the last dir to stdout on exit (to use for cd)",
 				Value:   false,
 			},
+			&cli.StringFlag{
+				Name:    "config-file",
+				Aliases: []string{"c"},
+				Usage:   "Specify the path to a different config file",
+				Value:   "", // Default to the blank string indicating non-usage of flag
+			},
 		},
 		Action: func(c *cli.Context) error {
 			// If no args are called along with "spf" use current dir
@@ -74,6 +78,18 @@ func Run(content embed.FS) {
 				path = c.Args().First()
 			}
 
+			// Setting the config file path
+			configFileArg := c.String("config-file")
+
+			// Validate the config file exists
+			if configFileArg != "" {
+				if _, err := os.Stat(variable.ConfigFile); err != nil {
+					log.Fatalf("Error: While reading config file '%s' from arguement : %v", configFileArg, err)
+				} else {
+					variable.ConfigFile = configFileArg
+				}
+			}
+
 			InitConfigFile()
 
 			err := InitTrash()
@@ -88,6 +104,8 @@ func Run(content embed.FS) {
 
 			firstUse := checkFirstUse()
 
+			go CheckForUpdates()
+
 			p := tea.NewProgram(internal.InitialModel(path, firstUse, hasTrash), tea.WithAltScreen(), tea.WithMouseCellMotion())
 			if _, err := p.Run(); err != nil {
 				log.Fatalf("Alas, there's been an error: %v", err)
@@ -97,7 +115,6 @@ func Run(content embed.FS) {
 				fmt.Println(variable.LastDir)
 			}
 
-			CheckForUpdates()
 			return nil
 		},
 	}
@@ -123,7 +140,6 @@ func InitConfigFile() {
 
 	// Create files
 	if err := createFiles(
-		variable.PinnedFile,
 		variable.ToggleDotFile,
 		variable.LogFile,
 		variable.ThemeFileVersion,
@@ -140,15 +156,20 @@ func InitConfigFile() {
 	if err := writeConfigFile(variable.HotkeysFile, internal.HotkeysTomlString); err != nil {
 		log.Fatalln("Error writing config file:", err)
 	}
+
+	if err := initJsonFile(variable.PinnedFile); err != nil {
+		log.Fatalln("Error initializing json file:", err)
+	}
 }
 
+// We are initializing these, but not sure if we are ever using them
 func InitTrash() error {
 	// Create trash directories
 	if runtime.GOOS != "darwin" {
 		err := createDirectories(
-			xdg.DataHome+variable.TrashDirectory,
-			xdg.DataHome+variable.TrashDirectoryFiles,
-			xdg.DataHome+variable.TrashDirectoryInfo,
+			variable.CustomTrashDirectory,
+			variable.CustomTrashDirectoryFiles,
+			variable.CustomTrashDirectoryInfo,
 		)
 		return err
 	}
@@ -193,13 +214,13 @@ func checkFirstUse() bool {
 	if _, err := os.Stat(file); os.IsNotExist(err) {
 		firstUse = true
 		if err := os.WriteFile(file, nil, 0644); err != nil {
-			log.Fatalln("failed to create file: %w", err)
+			log.Fatalf("Failed to create file: %v", err)
 		}
 	}
 	return firstUse
 }
 
-// Write data to the path file if it exists
+// Write data to the path file if it does not exists
 func writeConfigFile(path, data string) error {
 	if _, err := os.Stat(path); os.IsNotExist(err) {
 		if err := os.WriteFile(path, []byte(data), 0644); err != nil {
@@ -209,18 +230,38 @@ func writeConfigFile(path, data string) error {
 	return nil
 }
 
+func initJsonFile(path string) error {
+	if _, err := os.Stat(path); os.IsNotExist(err) {
+		if err := os.WriteFile(path, []byte("null"), 0644); err != nil {
+			return fmt.Errorf("failed to initialize json file %s: %w", path, err)
+		}
+	}
+	return nil
+}
+
+func writeLastCheckTime(t time.Time) {
+	err := os.WriteFile(variable.LastCheckVersion, []byte(t.Format(time.RFC3339)), 0644)
+	if err != nil {
+		slog.Error("Error writing LastCheckVersion file", "error", err)
+	}
+}
+
 // Check for the need of updates if AutoCheckUpdate is on, if its the first time
 // that version is checked or if has more than 24h since the last version check,
 // look into the repo if  there's any more recent version
+// Todo : This is too big of a function. Refactor it to displayUpdateNotification, fetchLatestVersion,
+// shouldCheckForUpdates, chucks
 func CheckForUpdates() {
 	var Config internal.ConfigType
 
 	// Get AutoCheck flag from configuration files
+
+	// Todo : We are reading the config file here, and also in the loadConfigFile functions
+	// This needs to be fixed.
 	data, err := os.ReadFile(variable.ConfigFile)
 	if err != nil {
 		log.Fatalf("Config file doesn't exist: %v", err)
 	}
-
 	err = toml.Unmarshal(data, &Config)
 	if err != nil {
 		log.Fatalf("Error decoding config file ( your config file may be misconfigured ): %v", err)
@@ -230,25 +271,43 @@ func CheckForUpdates() {
 		return
 	}
 
+	// Get current time in UTC
+	currentTime := time.Now().UTC()
+
 	// Check last time the version was checked
-	lastTime, err := readLastTimeCheckVersionFromFile(variable.LastCheckVersion)
-	if err != nil && !os.IsNotExist(err) {
-		fmt.Println("Error reading from file:", err)
-		return
+	content, err := os.ReadFile(variable.LastCheckVersion)
+
+	// Default to zero time if file doesn't exist, is empty, or has errors
+	lastTime := time.Time{}
+
+	if err == nil && len(content) > 0 {
+		parsedTime, parseErr := time.Parse(time.RFC3339, string(content))
+		if parseErr == nil {
+			lastTime = parsedTime.UTC()
+		} else {
+			// Let the time stay as zero initialized value
+			slog.Error("Error parsing time from LastCheckVersion file. Setting last time to zero", "error", parseErr)
+		}
 	}
 
-	currentTime := time.Now()
-
 	if lastTime.IsZero() || currentTime.Sub(lastTime) >= 24*time.Hour {
-		resp, err := http.Get(variable.LatestVersionURL)
+		// We would make sure to update the file in all return paths
+		defer func() {
+			writeLastCheckTime(currentTime)
+		}()
+		client := &http.Client{
+			Timeout: 5 * time.Second,
+		}
+		resp, err := client.Get(variable.LatestVersionURL)
 		if err != nil {
-			fmt.Println("Error checking for updates:", err)
+			slog.Error("Error checking for updates:", "error", err)
 			return
 		}
 		defer resp.Body.Close()
 
 		body, err := io.ReadAll(resp.Body)
 		if err != nil {
+			slog.Error("Error reading response body", "error", err)
 			return
 		}
 
@@ -258,11 +317,18 @@ func CheckForUpdates() {
 
 		var release GitHubRelease
 		if err := json.Unmarshal(body, &release); err != nil {
+			// Update the timestamp file even if JSON parsing fails
+			slog.Error("Error parsing JSON from Github", "error", err)
 			return
 		}
 
-		//Check if the local version is outdated
-		if versionToNumber(release.TagName) > versionToNumber(variable.CurrentVersion) {
+		// Check if the local version is outdated
+		res, err := versionCompare(release.TagName, variable.CurrentVersion)
+		if err != nil {
+			slog.Error("Error while trying to compare versions", "error", err)
+			return
+		}
+		if res > 0 {
 			fmt.Println(lipgloss.NewStyle().Foreground(lipgloss.Color("#FF69E1")).Render("┃ ") +
 				lipgloss.NewStyle().Foreground(lipgloss.Color("#FFBA52")).Bold(true).Render("A new version ") +
 				lipgloss.NewStyle().Foreground(lipgloss.Color("#00FFF2")).Bold(true).Italic(true).Render(release.TagName) +
@@ -271,54 +337,5 @@ func CheckForUpdates() {
 			fmt.Printf(lipgloss.NewStyle().Foreground(lipgloss.Color("#FF69E1")).Render("┃ ")+"Please update.\n┏\n\n      => %s\n\n", variable.LatestVersionGithub)
 			fmt.Printf("                                                               ┛\n")
 		}
-
-		timeStr := currentTime.Format(time.RFC3339)
-		err = writeToFile(variable.LastCheckVersion, timeStr)
-		if err != nil {
-			log.Println("Error writing to file:", err)
-			return
-		}
 	}
 }
-
-// Convert version string to number
-func versionToNumber(version string) int {
-	version = strings.ReplaceAll(version, "v", "")
-	version = strings.ReplaceAll(version, ".", "")
-
-	num, _ := strconv.Atoi(version)
-	return num
-}
-
-// Check the last time the version file was checked
-func readLastTimeCheckVersionFromFile(filename string) (time.Time, error) {
-	content, err := os.ReadFile(filename)
-	if err != nil {
-		return time.Time{}, err
-	}
-	if len(content) == 0 {
-		return time.Time{}, nil
-	}
-	lastTime, err := time.Parse(time.RFC3339, string(content))
-	if err != nil {
-		return time.Time{}, err
-	}
-
-	return lastTime, nil
-}
-
-// Write content to filename
-func writeToFile(filename, content string) error {
-	file, err := os.OpenFile(filename, os.O_WRONLY|os.O_CREATE, 0644)
-	if err != nil {
-		return err
-	}
-	defer file.Close()
-
-	_, err = file.WriteString(content)
-	if err != nil {
-		return err
-	}
-
-	return nil
-}
