@@ -5,10 +5,11 @@ import (
 	"encoding/hex"
 	"fmt"
 	"io"
-	"log"
+	"log/slog"
 	"os"
 	"path/filepath"
 	"regexp"
+	"slices"
 	"sort"
 	"strconv"
 	"strings"
@@ -41,127 +42,115 @@ func returnFocusType(focusPanel focusPanelType) filePanelFocusType {
 	return secondFocus
 }
 
-func returnFolderElement(location string, displayDotFile bool, sortOptions sortOptionsModelData) (directoryElement []element) {
-
-	files, err := os.ReadDir(location)
-	if len(files) == 0 {
+func returnDirElement(location string, displayDotFile bool, sortOptions sortOptionsModelData) (directoryElement []element) {
+	dirEntries, err := os.ReadDir(location)
+	if err != nil {
+		slog.Error("Error while return folder element function", "error", err)
 		return directoryElement
 	}
 
-	if err != nil {
-		outPutLog("Return folder element function error", err)
+	dirEntries = slices.DeleteFunc(dirEntries, func(e os.DirEntry) bool {
+		// Entries not needed to be considered
+		_, err := e.Info()
+		return err != nil || (strings.HasPrefix(e.Name(), ".") && !displayDotFile)
+	})
+
+	// No files/directoes to process
+	if len(dirEntries) == 0 {
+		return directoryElement
 	}
 
 	// Sort files
 	var order func(i, j int) bool
 	reversed := sortOptions.reversed
 
+	// Todo : These strings should not be hardcoded here, but defined as constants
 	switch sortOptions.options[sortOptions.selected] {
 	case "Name":
 		order = func(i, j int) bool {
-			_, errI := os.ReadDir(location + "/" + files[i].Name())
-			_, errJ := os.ReadDir(location + "/" + files[j].Name())
-			if (files[i].IsDir() || errI == nil) && (!files[j].IsDir() && errJ != nil) {
-				return true
-			}
-			if (!files[i].IsDir() && errI != nil) && (files[j].IsDir() || errJ == nil) {
-				return false
+			// One of them is a directory, and other is not
+			if dirEntries[i].IsDir() != dirEntries[j].IsDir() {
+				return dirEntries[i].IsDir()
 			}
 			if Config.CaseSensitiveSort {
-				return files[i].Name() < files[j].Name() != reversed
+				return dirEntries[i].Name() < dirEntries[j].Name() != reversed
 			} else {
-				return strings.ToLower(files[i].Name()) < strings.ToLower(files[j].Name()) != reversed
+				return strings.ToLower(dirEntries[i].Name()) < strings.ToLower(dirEntries[j].Name()) != reversed
 			}
 		}
 	case "Size":
 		order = func(i, j int) bool {
 			// Directories at the top sorted by direct child count (not recursive)
 			// Files sorted by size
-			fileInfoI, _ := files[i].Info()
-			fileInfoJ, _ := files[j].Info()
-			_, errI := os.ReadDir(location + "/" + files[i].Name())
-			_, errJ := os.ReadDir(location + "/" + files[j].Name())
 
-			if (files[i].IsDir() || errI == nil) && (!files[j].IsDir() && errJ != nil) {
-				return true
+			// One of them is a directory, and other is not
+			if dirEntries[i].IsDir() != dirEntries[j].IsDir() {
+				return dirEntries[i].IsDir()
 			}
-			if (!files[i].IsDir() && errI != nil) && (files[j].IsDir() || errJ == nil) {
-				return false
-			}
-			if files[i].IsDir() && files[j].IsDir() {
-				filesI, err := os.ReadDir(filepath.Join(location, files[i].Name()))
+
+			// This needs to be improved, and we should sort by actual size only
+			// Repeated recursive read would be slow, so we could cache
+			if dirEntries[i].IsDir() && dirEntries[j].IsDir() {
+				filesI, err := os.ReadDir(filepath.Join(location, dirEntries[i].Name()))
+				// No need of early return, we only call len() on filesI, so nil would
+				// just result in 0
 				if err != nil {
-					outPutLog("Error when reading directory", err)
+					slog.Error("Error when reading directory during sort", "error", err)
 				}
-				filesJ, err := os.ReadDir(filepath.Join(location, files[j].Name()))
+				filesJ, err := os.ReadDir(filepath.Join(location, dirEntries[j].Name()))
 				if err != nil {
-					outPutLog("Error when reading directory", err)
+					slog.Error("Error when reading directory during sort", "error", err)
 				}
 				return len(filesI) < len(filesJ) != reversed
+			} else {
+				// No need for err check, we already filtered out dirEntries with err != nil in Info() call
+				fileInfoI, _ := dirEntries[i].Info()
+				fileInfoJ, _ := dirEntries[j].Info()
+				return fileInfoI.Size() < fileInfoJ.Size() != reversed
 			}
-			return fileInfoI.Size() < fileInfoJ.Size() != reversed
+
 		}
 	case "Date Modified":
 		order = func(i, j int) bool {
-			fileInfoI, _ := files[i].Info()
-			fileInfoJ, _ := files[j].Info()
+			// No need for err check, we already filtered out dirEntries with err != nil in Info() call
+			fileInfoI, _ := dirEntries[i].Info()
+			fileInfoJ, _ := dirEntries[j].Info()
 			return fileInfoI.ModTime().After(fileInfoJ.ModTime()) != reversed
 		}
 	}
 
-	sort.Slice(files, order)
-
-	for _, item := range files {
-		fileInfo, err := item.Info()
-		if err != nil {
-			continue
-		}
-
-		if !displayDotFile && strings.HasPrefix(fileInfo.Name(), ".") {
-			continue
-		}
-		if fileInfo == nil {
-			continue
-		}
-		newElement := element{
+	sort.Slice(dirEntries, order)
+	for _, item := range dirEntries {
+		directoryElement = append(directoryElement, element{
 			name:      item.Name(),
 			directory: item.IsDir(),
-		}
-		if location == "/" {
-			newElement.location = location + item.Name()
-		} else {
-			newElement.location = filepath.Join(location, item.Name())
-		}
-		directoryElement = append(directoryElement, newElement)
+			location:  filepath.Join(location, item.Name()),
+		})
 	}
-
 	return directoryElement
 }
 
-func returnFolderElementBySearchString(location string, displayDotFile bool, searchString string) (folderElement []element) {
+func returnDirElementBySearchString(location string, displayDotFile bool, searchString string) (dirElement []element) {
 
 	items, err := os.ReadDir(location)
 	if err != nil {
-		outPutLog("Return folder element function error", err)
+		slog.Error("Error while return folder element function", "error", err)
+		return []element{}
 	}
 
 	folderElementMap := map[string]element{}
 	fileAndDirectories := []string{}
 
 	for _, item := range items {
-		fileInfo, _ := item.Info()
+		fileInfo, err := item.Info()
+		if err != nil {
+			continue
+		}
 		if !displayDotFile && strings.HasPrefix(fileInfo.Name(), ".") {
 			continue
 		}
 
-		if fileInfo == nil {
-			continue
-		}
-
-		folderElementLocation := location + "/" + item.Name()
-		if location == "/" {
-			folderElementLocation = location + item.Name()
-		}
+		folderElementLocation := filepath.Join(location, item.Name())
 
 		fileAndDirectories = append(fileAndDirectories, item.Name())
 		folderElementMap[item.Name()] = element{
@@ -169,29 +158,24 @@ func returnFolderElementBySearchString(location string, displayDotFile bool, sea
 			directory: item.IsDir(),
 			location:  folderElementLocation,
 		}
-
 	}
-
-	var options = fzf.DefaultOptions()
-	var hayStack = fileAndDirectories
-	var myFzf = fzf.New(hayStack, options)
-	var result fzf.SearchResult
-	myFzf.Search(searchString)
-	result = <-myFzf.GetResultChannel()
-	myFzf.End()
-
-	for _, item := range result.Matches {
+	// https://github.com/reinhrst/fzf-lib/blob/main/core.go#L43
+	// No sorting needed. fzf.DefaultOptions() already return values ordered on Score
+	for _, item := range fzfSearch(searchString, fileAndDirectories) {
 		resultItem := folderElementMap[item.Key]
-		resultItem.matchRate = float64(item.Score)
-		folderElement = append(folderElement, resultItem)
+		dirElement = append(dirElement, resultItem)
 	}
 
-	// Sort folders and files by match rate
-	sort.Slice(folderElement, func(i, j int) bool {
-		return folderElement[i].matchRate > folderElement[j].matchRate
-	})
+	return dirElement
+}
 
-	return folderElement
+// Returning a string slice causes inefficiency in current usage
+func fzfSearch(query string, source []string) []fzf.MatchResult {
+	fzfSearcher := fzf.New(source, fzf.DefaultOptions())
+	fzfSearcher.Search(query)
+	fzfResults := <-fzfSearcher.GetResultChannel()
+	fzfSearcher.End()
+	return fzfResults.Matches
 }
 
 func panelElementHeight(mainPanelHeight int) int {
@@ -202,6 +186,7 @@ func bottomElementHeight(bottomElementHeight int) int {
 	return bottomElementHeight - 5
 }
 
+// Todo : replace usage of this with slices.contains
 func arrayContains(s []string, str string) bool {
 	for _, v := range s {
 		if v == str {
@@ -211,11 +196,10 @@ func arrayContains(s []string, str string) bool {
 	return false
 }
 
-func outPutLog(values ...interface{}) {
-	log.SetOutput(logOutput)
-	for _, value := range values {
-		log.Println(value)
-	}
+// Todo : Eventually we want to remove all such usage that can result in app exiting abruptly
+func LogAndExit(msg string, values ...any) {
+	slog.Error(msg, values...)
+	os.Exit(1)
 }
 
 func removeElementByValue(slice []string, value string) []string {
@@ -284,6 +268,7 @@ func renameIfDuplicate(destination string) (string, error) {
 	}
 }
 
+// Todo : Move this model related function to the right file. Should not be in functions file.
 func (m *model) returnMetaData() {
 	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
 	cursor := panel.cursor
@@ -334,7 +319,7 @@ func (m *model) returnMetaData() {
 	}
 
 	if err != nil {
-		outPutLog("Return meta data function get file state error", err)
+		slog.Error("Error while return meta data function get file state", "error", err)
 	}
 
 	if fileInfo.IsDir() {
@@ -351,17 +336,17 @@ func (m *model) returnMetaData() {
 
 	checkIsSymlinked, err := os.Lstat(filePath)
 	if err != nil {
-		outPutLog("err when getting file info", err)
+		slog.Error("Error when getting file info", "error", err)
 		return
 	}
 
-	if Config.Metadata && checkIsSymlinked.Mode()&os.ModeSymlink == 0 {
+	if Config.Metadata && checkIsSymlinked.Mode()&os.ModeSymlink == 0 && et != nil {
 
 		fileInfos := et.ExtractMetadata(filePath)
 
 		for _, fileInfo := range fileInfos {
 			if fileInfo.Err != nil {
-				outPutLog("Return meta data function error", fileInfo, fileInfo.Err)
+				slog.Error("Error while return metadata function", "fileInfo", fileInfo, "error", fileInfo.Err)
 				continue
 			}
 
@@ -380,7 +365,7 @@ func (m *model) returnMetaData() {
 			// Calculate MD5 checksum
 			checksum, err := calculateMD5Checksum(filePath)
 			if err != nil {
-				outPutLog("Error calculating MD5 checksum", err)
+				slog.Error("Error calculating MD5 checksum", "error", err)
 			} else {
 				md5Data := [2]string{"MD5Checksum", checksum}
 				m.fileMetaData.metaData = append(m.fileMetaData.metaData, md5Data)
@@ -415,9 +400,9 @@ func calculateMD5Checksum(filePath string) (string, error) {
 // Get directory total size
 func dirSize(path string) int64 {
 	var size int64
-	filepath.WalkDir(path, func(_ string, entry os.DirEntry, err error) error {
+	err := filepath.WalkDir(path, func(_ string, entry os.DirEntry, err error) error {
 		if err != nil {
-			outPutLog("Dir size function error", err)
+			slog.Error("Dir size function error", "error", err)
 		}
 		if !entry.IsDir() {
 			info, err := entry.Info()
@@ -427,6 +412,9 @@ func dirSize(path string) int64 {
 		}
 		return err
 	})
+	if err != nil {
+		slog.Error("errors during WalkDir", "error", err)
+	}
 	return size
 }
 
