@@ -1,10 +1,13 @@
 package internal
 
 import (
+	"errors"
+	"fmt"
+	"log/slog"
 	"os"
 	"os/exec"
-	"path"
 	"path/filepath"
+	"runtime"
 	"strings"
 	"time"
 
@@ -18,13 +21,13 @@ import (
 
 // Create a file in the currently focus file panel
 func (m *model) panelCreateNewFile() {
-	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
+	panel := &m.fileModel.filePanels[m.filePanelFocusIndex]
 	ti := textinput.New()
 	ti.Cursor.Style = modalCursorStyle
 	ti.Cursor.TextStyle = modalStyle
 	ti.TextStyle = modalStyle
 	ti.Cursor.Blink = true
-	ti.Placeholder = "Add \"/\" represent Transcend folder at the end"
+	ti.Placeholder = "Add \"" + string(filepath.Separator) + "\" transcend folders"
 	ti.PlaceholderStyle = modalStyle
 	ti.Focus()
 	ti.CharLimit = 156
@@ -35,14 +38,12 @@ func (m *model) panelCreateNewFile() {
 	m.typingModal.textInput = ti
 	m.firstTextInput = true
 
-	m.fileModel.filePanels[m.filePanelFocusIndex] = panel
-
 }
 
 func (m *model) IsRenamingConflicting() bool {
-	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
+	panel := &m.fileModel.filePanels[m.filePanelFocusIndex]
 	oldPath := panel.element[panel.cursor].location
-	newPath := panel.location + "/" + panel.rename.Value()
+	newPath := filepath.Join(panel.location, panel.rename.Value())
 
 	if oldPath == newPath {
 		return false
@@ -70,10 +71,17 @@ func (m *model) warnModalForRenaming() {
 
 // Rename file where the cusror is located
 func (m *model) panelItemRename() {
-	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
+	panel := &m.fileModel.filePanels[m.filePanelFocusIndex]
 	if len(panel.element) == 0 {
 		return
 	}
+
+	cursorPos := strings.LastIndex(panel.element[panel.cursor].name, ".")
+	nameLen := len(panel.element[panel.cursor].name)
+	if cursorPos == -1 || cursorPos == 0 && nameLen > 0 || panel.element[panel.cursor].directory {
+		cursorPos = nameLen
+	}
+
 	ti := textinput.New()
 	ti.Cursor.Style = filePanelCursorStyle
 	ti.Cursor.TextStyle = filePanelStyle
@@ -83,6 +91,7 @@ func (m *model) panelItemRename() {
 	ti.Placeholder = "New name"
 	ti.PlaceholderStyle = modalStyle
 	ti.SetValue(panel.element[panel.cursor].name)
+	ti.SetCursor(cursorPos)
 	ti.Focus()
 	ti.CharLimit = 156
 	ti.Width = m.fileModel.width - 4
@@ -91,11 +100,10 @@ func (m *model) panelItemRename() {
 	panel.renaming = true
 	m.firstTextInput = true
 	panel.rename = ti
-	m.fileModel.filePanels[m.filePanelFocusIndex] = panel
 }
 
 func (m *model) deleteItemWarn() {
-	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
+	panel := &m.fileModel.filePanels[m.filePanelFocusIndex]
 	if !((panel.panelMode == selectMode && len(panel.selected) != 0) || (panel.panelMode == browserMode)) {
 		return
 	}
@@ -129,7 +137,7 @@ func (m *model) deleteItemWarn() {
 // Move file or directory to the trash can
 func (m *model) deleteSingleItem() {
 	id := shortuuid.New()
-	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
+	panel := &m.fileModel.filePanels[m.filePanelFocusIndex]
 
 	if len(panel.element) == 0 {
 		return
@@ -176,12 +184,11 @@ func (m *model) deleteSingleItem() {
 			panel.cursor = len(panel.element) - 1
 		}
 	}
-	m.fileModel.filePanels[m.filePanelFocusIndex] = panel
 }
 
 // Move file or directory to the trash can
-func (m model) deleteMultipleItems() {
-	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
+func (m *model) deleteMultipleItems() {
+	panel := &m.fileModel.filePanels[m.filePanelFocusIndex]
 	if len(panel.selected) != 0 {
 		id := shortuuid.New()
 		prog := progress.New(generateGradientColor())
@@ -221,7 +228,7 @@ func (m model) deleteMultipleItems() {
 				p.state = failure
 				message.processNewState = p
 				channel <- message
-				outPutLog("Delete multiple item function error", err)
+				slog.Error("Error while delete multiple item function", "error", err)
 				m.processBarModel.process[id] = p
 				break
 			} else {
@@ -247,7 +254,7 @@ func (m model) deleteMultipleItems() {
 // Completely delete file or folder (Not move to the trash can)
 func (m *model) completelyDeleteSingleItem() {
 	id := shortuuid.New()
-	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
+	panel := &m.fileModel.filePanels[m.filePanelFocusIndex]
 
 	if len(panel.element) == 0 {
 		return
@@ -275,7 +282,7 @@ func (m *model) completelyDeleteSingleItem() {
 
 	err := os.RemoveAll(panel.element[panel.cursor].location)
 	if err != nil {
-		outPutLog("Completely delete single item function remove file error", err)
+		slog.Error("Error while completely delete single item function remove file", "error", err)
 	}
 
 	if err != nil {
@@ -298,12 +305,11 @@ func (m *model) completelyDeleteSingleItem() {
 			panel.cursor = len(panel.element) - 1
 		}
 	}
-	m.fileModel.filePanels[m.filePanelFocusIndex] = panel
 }
 
 // Completely delete all file or folder from clipboard (Not move to the trash can)
-func (m model) completelyDeleteMultipleItems() {
-	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
+func (m *model) completelyDeleteMultipleItems() {
+	panel := &m.fileModel.filePanels[m.filePanelFocusIndex]
 	if len(panel.selected) != 0 {
 		id := shortuuid.New()
 		prog := progress.New(generateGradientColor())
@@ -338,14 +344,14 @@ func (m model) completelyDeleteMultipleItems() {
 			}
 			err := os.RemoveAll(filePath)
 			if err != nil {
-				outPutLog("Completely delete multiple item function remove file error", err)
+				slog.Error("Error while completely delete multiple item function remove file", "error", err)
 			}
 
 			if err != nil {
 				p.state = failure
 				message.processNewState = p
 				channel <- message
-				outPutLog("Completely delete multiple item function error", err)
+				slog.Error("Error while completely delete multiple item function", "error", err)
 				m.processBarModel.process[id] = p
 				break
 			} else {
@@ -368,151 +374,65 @@ func (m model) completelyDeleteMultipleItems() {
 	panel.selected = panel.selected[:0]
 }
 
-// Copy directory or file
-func (m *model) copySingleItem() {
-	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
-	m.copyItems.cut = false
-	m.copyItems.items = m.copyItems.items[:0]
+// Copy directory or file's path to superfile's clipboard
+// set cut to true/false accordingly
+func (m *model) copySingleItem(cut bool) {
+	panel := &m.fileModel.filePanels[m.filePanelFocusIndex]
+	m.copyItems.reset(cut)
 	if len(panel.element) == 0 {
 		return
 	}
+	slog.Debug("handle_file_operations.copySingleItem", "cut", cut,
+		"panel location", panel.element[panel.cursor].location)
 	m.copyItems.items = append(m.copyItems.items, panel.element[panel.cursor].location)
-	fileInfo, err := os.Stat(panel.element[panel.cursor].location)
-	if os.IsNotExist(err) {
-		m.copyItems.items = m.copyItems.items[:0]
-		return
-	}
-	if err != nil {
-		outPutLog("Copy single item get file state error", panel.element[panel.cursor].location, err)
-	}
-
-	if !fileInfo.IsDir() && float64(fileInfo.Size())/(1024*1024) < 250 {
-		fileContent, err := os.ReadFile(panel.element[panel.cursor].location)
-
-		if err != nil {
-			outPutLog("Copy single item read file error", panel.element[panel.cursor].location, err)
-		}
-
-		if err := clipboard.WriteAll(string(fileContent)); err != nil {
-			outPutLog("Copy single item write file error", panel.element[panel.cursor].location, err)
-		}
-	}
-	m.fileModel.filePanels[m.filePanelFocusIndex] = panel
 }
 
-// Copy all selected file or directory to the clipboard
-func (m *model) copyMultipleItem() {
-	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
-	m.copyItems.cut = false
-	m.copyItems.items = m.copyItems.items[:0]
+// Copy all selected file or directory's paths to the clipboard
+func (m *model) copyMultipleItem(cut bool) {
+	panel := &m.fileModel.filePanels[m.filePanelFocusIndex]
+	m.copyItems.reset(cut)
 	if len(panel.selected) == 0 {
 		return
 	}
+	slog.Debug("handle_file_operations.copyMultipleItem", "cut", cut,
+		"panel selected files", panel.selected)
 	m.copyItems.items = panel.selected
-	fileInfo, err := os.Stat(panel.selected[0])
-	if os.IsNotExist(err) {
-		return
-	}
-	if err != nil {
-		outPutLog("Copy multiple item function get file state error", panel.selected[0], err)
-	}
-
-	if !fileInfo.IsDir() && float64(fileInfo.Size())/(1024*1024) < 250 {
-		fileContent, err := os.ReadFile(panel.selected[0])
-
-		if err != nil {
-			outPutLog("Copy multiple item function read file error", err)
-		}
-
-		if err := clipboard.WriteAll(string(fileContent)); err != nil {
-			outPutLog("Copy multiple item function write file to clipboard error", err)
-		}
-	}
-	m.fileModel.filePanels[m.filePanelFocusIndex] = panel
-}
-
-// Cut directory or file
-func (m *model) cutSingleItem() {
-	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
-	m.copyItems.cut = true
-	m.copyItems.items = m.copyItems.items[:0]
-	if len(panel.element) == 0 {
-		return
-	}
-	m.copyItems.items = append(m.copyItems.items, panel.element[panel.cursor].location)
-	fileInfo, err := os.Stat(panel.element[panel.cursor].location)
-	if os.IsNotExist(err) {
-		m.copyItems.items = m.copyItems.items[:0]
-		return
-	}
-	if err != nil {
-		outPutLog("Cut single item get file state error", panel.element[panel.cursor].location, err)
-	}
-
-	if !fileInfo.IsDir() && float64(fileInfo.Size())/(1024*1024) < 250 {
-		fileContent, err := os.ReadFile(panel.element[panel.cursor].location)
-
-		if err != nil {
-			outPutLog("Cut single item read file error", panel.element[panel.cursor].location, err)
-		}
-
-		if err := clipboard.WriteAll(string(fileContent)); err != nil {
-			outPutLog("Cut single item write file error", panel.element[panel.cursor].location, err)
-		}
-	}
-	m.fileModel.filePanels[m.filePanelFocusIndex] = panel
-}
-
-// Cut all selected file or directory to the clipboard
-func (m *model) cutMultipleItem() {
-	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
-	m.copyItems.cut = true
-	m.copyItems.items = m.copyItems.items[:0]
-	if len(panel.selected) == 0 {
-		return
-	}
-	m.copyItems.items = panel.selected
-	fileInfo, err := os.Stat(panel.selected[0])
-	if os.IsNotExist(err) {
-		return
-	}
-	if err != nil {
-		outPutLog("Copy multiple item function get file state error", panel.selected[0], err)
-	}
-
-	if !fileInfo.IsDir() && float64(fileInfo.Size())/(1024*1024) < 250 {
-		fileContent, err := os.ReadFile(panel.selected[0])
-
-		if err != nil {
-			outPutLog("Copy multiple item function read file error", err)
-		}
-
-		if err := clipboard.WriteAll(string(fileContent)); err != nil {
-			outPutLog("Copy multiple item function write file to clipboard error", err)
-		}
-	}
-	m.fileModel.filePanels[m.filePanelFocusIndex] = panel
 }
 
 // Paste all clipboard items
-func (m model) pasteItem() {
-	id := shortuuid.New()
-	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
-
+func (m *model) pasteItem() {
 	if len(m.copyItems.items) == 0 {
 		return
 	}
 
+	id := shortuuid.New()
+	panel := &m.fileModel.filePanels[m.filePanelFocusIndex]
 	totalFiles := 0
 
 	for _, folderPath := range m.copyItems.items {
+		// Todo : Fix this. This is inefficient
+		// In case of a cut operations for a directory with a lot of files
+		// we are unnecessarily walking the whole directory recursively
+		// while os will just perform a rename
+		// So instead of few operations this will cause the cut paste
+		// to read the whole directory recursively
+		// we should avoid doing this.
+		// Although this allows us a more detailed progress tracking
+		// this make the copy/cut more inefficient
+		// instead, we could just track progress based on total items in
+		// m.copyItems.items
+		// efficiency should be prioritized over more detailed feedback.
 		count, err := countFiles(folderPath)
 		if err != nil {
+			slog.Error("mode.pasteItem - Error in countFiles", "error", err)
 			continue
 		}
 		totalFiles += count
 	}
 
+	slog.Debug("model.pasteItem", "items", m.copyItems.items, "cut", m.copyItems.cut,
+		"totalFiles", totalFiles, "panel location", panel.location)
+
 	prog := progress.New(generateGradientColor())
 	prog.PercentageStyle = footerStyle
 
@@ -553,93 +473,125 @@ func (m model) pasteItem() {
 
 		errMessage := "cut item error"
 		if m.copyItems.cut && !isExternalDiskPath(filePath) {
-			err = moveElement(filePath, filepath.Join(panel.location, path.Base(filePath)))
+			err = moveElement(filePath, filepath.Join(panel.location, filepath.Base(filePath)))
 		} else {
-			newModel, err := pasteDir(filePath, filepath.Join(panel.location, path.Base(filePath)), id, m)
+			err = pasteDir(filePath, filepath.Join(panel.location, filepath.Base(filePath)), id, m)
 			if err != nil {
 				errMessage = "paste item error"
-			}
-			m = newModel
-			if m.copyItems.cut {
-				os.RemoveAll(filePath)
+			} else {
+				// Todo : These error cases are hard to test. We have to somehow make the paste operations fail,
+				// which is time consuming and manual. We should test these with automated testcases
+				if m.copyItems.cut {
+					os.RemoveAll(filePath)
+				}
 			}
 		}
 		p = m.processBarModel.process[id]
 		if err != nil {
+			slog.Debug("model.pasteItem - paste failure", "error", err,
+				"current item", filePath, "errMessage", errMessage)
 			p.state = failure
 			message.processNewState = p
 			channel <- message
-			outPutLog(errMessage, err)
+			slog.Error(errMessage, "error", err)
 			m.processBarModel.process[id] = p
 			break
 		}
 	}
 
-	p.state = successful
-	p.done = totalFiles
-	p.doneTime = time.Now()
+	if p.state != failure {
+		p.state = successful
+		p.done = totalFiles
+		p.doneTime = time.Now()
+	}
 	message.processNewState = p
 	channel <- message
 
 	m.processBarModel.process[id] = p
-	m.copyItems.cut = false
+	// Reset after paste is done. Only in case of cut
+	// because current items in clipboard are deleted now
+	if m.copyItems.cut {
+		m.copyItems.reset(false)
+	}
 }
 
-// Extrach compress file
-func (m model) extractFile() {
+// Extract compressed file
+// Todo : err should be returned and properly handled by the caller
+func (m *model) extractFile() {
 	var err error
-	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
+	panel := &m.fileModel.filePanels[m.filePanelFocusIndex]
 	ext := strings.ToLower(filepath.Ext(panel.element[panel.cursor].location))
-	outputDir := fileNameWithoutExtension(panel.element[panel.cursor].location)
-	outputDir, err = renameIfDuplicate(outputDir)
-
-	if err != nil {
-		outPutLog("Error extract file when create new directory", err)
+	if !isExensionExtractable(ext) {
+		slog.Error(fmt.Sprintf("Error unexpected file extension type: %s", ext), "error", errors.ErrUnsupported)
+		return
 	}
 
-	switch ext {
-	case ".zip":
-		os.MkdirAll(outputDir, 0755)
-		err = unzip(panel.element[panel.cursor].location, outputDir)
-		if err != nil {
-			outPutLog("Error extract file,", err)
-		}
-	default:
-		os.MkdirAll(outputDir, 0755)
-		err = extractCompressFile(panel.element[panel.cursor].location, outputDir)
-		if err != nil {
-			outPutLog("Error extract file,", err)
-		}
+	outputDir := fileNameWithoutExtension(panel.element[panel.cursor].location)
+	outputDir, err = renameIfDuplicate(outputDir)
+	if err != nil {
+		slog.Error("Error extract file when create new directory", "error", err)
+		return
+	}
+
+	err = os.MkdirAll(outputDir, 0755)
+	if err != nil {
+		slog.Error("Error while making directory for extracting files", "error", err)
+		return
+	}
+	err = extractCompressFile(panel.element[panel.cursor].location, outputDir)
+	if err != nil {
+		slog.Error("Error extract file", "error", err)
+		return
 	}
 }
 
 // Compress file or directory
-func (m model) compressFile() {
-	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
+func (m *model) compressFile() {
+	panel := &m.fileModel.filePanels[m.filePanelFocusIndex]
 	fileName := filepath.Base(panel.element[panel.cursor].location)
 
 	zipName := strings.TrimSuffix(fileName, filepath.Ext(fileName)) + ".zip"
 	zipName, err := renameIfDuplicate(zipName)
 
 	if err != nil {
-		outPutLog("Error compress file when rename duplicate", err)
+		slog.Error("Error in compressing files during rename duplicate", "error", err)
+		return
 	}
 
-	zipSource(panel.element[panel.cursor].location, filepath.Join(filepath.Dir(panel.element[panel.cursor].location), zipName))
+	err = zipSource(panel.element[panel.cursor].location, filepath.Join(filepath.Dir(panel.element[panel.cursor].location), zipName))
+	if err != nil {
+		slog.Error("Error in zipping files", "error", err)
+		// Although return is not needed here at the moment. This clarifies the intent of
+		// not continuing after the error even if any further code is added in this function later
+		return
+	}
 }
 
 // Open file with default editor
-func (m model) openFileWithEditor() tea.Cmd {
-	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
+func (m *model) openFileWithEditor() tea.Cmd {
+	panel := &m.fileModel.filePanels[m.filePanelFocusIndex]
 
 	editor := Config.Editor
 	if editor == "" {
 		editor = os.Getenv("EDITOR")
 	}
+
+	// Make sure there is an editor
+	// Todo : Move hardcoded strings to constants : "windows", and editors
 	if editor == "" {
-		editor = "nano"
+		if runtime.GOOS == "windows" {
+			editor = "notepad"
+		} else {
+			editor = "nano"
+		}
 	}
-	c := exec.Command(editor, panel.element[panel.cursor].location)
+
+	// Split the editor command into command and arguments
+	parts := strings.Fields(editor)
+	cmd := parts[0]
+	args := append(parts[1:], panel.element[panel.cursor].location)
+
+	c := exec.Command(cmd, args...)
 
 	return tea.ExecProcess(c, func(err error) tea.Msg {
 		return editorFinishedMsg{err}
@@ -647,31 +599,43 @@ func (m model) openFileWithEditor() tea.Cmd {
 }
 
 // Open directory with default editor
-func (m model) openDirectoryWithEditor() tea.Cmd {
-	editor := Config.Editor
+func (m *model) openDirectoryWithEditor() tea.Cmd {
+	// Todo : Move hardcoded strings to constants : "windows", and editors
+	editor := Config.DirEditor
+
 	if editor == "" {
-		editor = os.Getenv("EDITOR")
+		if runtime.GOOS == "windows" {
+			editor = "explorer"
+		} else if runtime.GOOS == "darwin" {
+			// open is command for MacOS Finder
+			editor = "open"
+		} else {
+			editor = "vi"
+		}
 	}
-	if editor == "" {
-		editor = "nano"
-	}
-	c := exec.Command(editor, m.fileModel.filePanels[m.filePanelFocusIndex].location)
+
+	// Split the editor command into command and arguments
+	parts := strings.Fields(editor)
+	cmd := parts[0]
+	args := append(parts[1:], m.fileModel.filePanels[m.filePanelFocusIndex].location)
+
+	c := exec.Command(cmd, args...)
 	return tea.ExecProcess(c, func(err error) tea.Msg {
 		return editorFinishedMsg{err}
 	})
 }
 
 // Copy file path
-func (m model) copyPath() {
-	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
+func (m *model) copyPath() {
+	panel := &m.fileModel.filePanels[m.filePanelFocusIndex]
 	if err := clipboard.WriteAll(panel.element[panel.cursor].location); err != nil {
-		outPutLog("Copy path error", panel.element[panel.cursor].location, err)
+		slog.Error("Error while copy path", "error", err)
 	}
 }
 
-func (m model) copyPWD() {
-	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
+func (m *model) copyPWD() {
+	panel := &m.fileModel.filePanels[m.filePanelFocusIndex]
 	if err := clipboard.WriteAll(panel.location); err != nil {
-		outPutLog("Copy present working directory error", panel.location, err)
+		slog.Error("Error while copy present working directory", "error", err)
 	}
 }
