@@ -1,7 +1,7 @@
 package internal
 
 import (
-	"fmt"
+	"log/slog"
 	"os"
 	"os/exec"
 	"path/filepath"
@@ -69,7 +69,7 @@ func (m *model) enterPanel() {
 	} else if !panel.element[panel.cursor].directory {
 		fileInfo, err := os.Lstat(panel.element[panel.cursor].location)
 		if err != nil {
-			outPutLog("err when getting file info", err)
+			slog.Error("Error while getting file info", "error", err)
 			return
 		}
 
@@ -103,7 +103,7 @@ func (m *model) enterPanel() {
 			cmd := exec.Command(dllpath, dllfile, panel.element[panel.cursor].location)
 			err = cmd.Start()
 			if err != nil {
-				outPutLog(fmt.Sprintf("err when open file with %s", openCommand), err)
+				slog.Error("Error while open file with", "error", err)
 			}
 
 			return
@@ -112,7 +112,7 @@ func (m *model) enterPanel() {
 		cmd := exec.Command(openCommand, panel.element[panel.cursor].location)
 		err = cmd.Start()
 		if err != nil {
-			outPutLog(fmt.Sprintf("err when open file with %s", openCommand), err)
+			slog.Error("Error while open file with", "error", err)
 		}
 
 	}
@@ -122,6 +122,11 @@ func (m *model) enterPanel() {
 
 // Switch to the directory where the sidebar cursor is located
 func (m *model) sidebarSelectDirectory() {
+	// We can't do this when we have only divider directories
+	// m.sidebarModel.directories[m.sidebarModel.cursor].location would point to a divider dir.
+	if m.sidebarModel.noActualDir() {
+		return
+	}
 	m.focusPanel = nonePanelFocus
 	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
 
@@ -166,8 +171,6 @@ func (m *model) singleItemSelect() {
 		}
 
 		m.fileModel.filePanels[m.filePanelFocusIndex] = panel
-	} else {
-		outPutLog("No elements to select or cursor out of bounds.")
 	}
 }
 
@@ -181,10 +184,10 @@ func (m *model) toggleDotFileController() {
 		newToggleDotFile = "true"
 		m.toggleDotFile = true
 	}
-  m.updatedToggleDotFile = true
+	m.updatedToggleDotFile = true
 	err := os.WriteFile(variable.ToggleDotFile, []byte(newToggleDotFile), 0644)
 	if err != nil {
-		outPutLog("Pinned folder function updatedData superfile data error", err)
+		slog.Error("Error while pinned folder function updatedData superfile data", "error", err)
 	}
 
 }
@@ -201,7 +204,7 @@ func (m *model) toggleFooterController() {
 	}
 	err := os.WriteFile(variable.ToggleFooter, []byte(newToggleFooterFile), 0644)
 	if err != nil {
-		outPutLog("Toggle footer function updatedData superfile data error", err)
+		slog.Error("Error while Toggle footer function updatedData superfile data", "error", err)
 	}
 	m.setFooterSize(m.fullHeight)
 
@@ -214,6 +217,7 @@ func (m *model) searchBarFocus() {
 		panel.searchBar.Blur()
 	} else {
 		panel.searchBar.Focus()
+		m.firstTextInput = true
 	}
 
 	// config search bar width
@@ -279,56 +283,56 @@ func (m *model) controlFilePanelListDown(wheel bool) {
 
 }
 
-func (m *model) controlFilePanelPgUp(){
-  panel := m.fileModel.filePanels[m.filePanelFocusIndex]
-  panlen := len(panel.element)
-  panHeight := panelElementHeight(m.mainPanelHeight)
-  panCenter := panHeight / 2     // For making sure the cursor is at the center of the panel
+func (m *model) controlFilePanelPgUp() {
+	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
+	panlen := len(panel.element)
+	panHeight := panelElementHeight(m.mainPanelHeight)
+	panCenter := panHeight / 2 // For making sure the cursor is at the center of the panel
 
-  if panlen == 0 {
-    return
-  }
+	if panlen == 0 {
+		return
+	}
 
-  if panHeight >= panlen {
-    panel.cursor = 0
-  } else {
-    if panel.cursor - panHeight <= 0 {
-      panel.cursor = 0
-      panel.render = 0
-    } else {
-      panel.cursor -= panHeight
-      panel.render = panel.cursor - panCenter
-      
-      if panel.render < 0 {
-        panel.render = 0
-      }
-    }
-  }
+	if panHeight >= panlen {
+		panel.cursor = 0
+	} else {
+		if panel.cursor-panHeight <= 0 {
+			panel.cursor = 0
+			panel.render = 0
+		} else {
+			panel.cursor -= panHeight
+			panel.render = panel.cursor - panCenter
+
+			if panel.render < 0 {
+				panel.render = 0
+			}
+		}
+	}
 
 	m.fileModel.filePanels[m.filePanelFocusIndex] = panel
 }
 
-func (m *model) controlFilePanelPgDown(){
-  panel := m.fileModel.filePanels[m.filePanelFocusIndex]
-  panlen := len(panel.element)
-  panHeight := panelElementHeight(m.mainPanelHeight)
-  panCenter := panHeight / 2     // For making sure the cursor is at the center of the panel
+func (m *model) controlFilePanelPgDown() {
+	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
+	panlen := len(panel.element)
+	panHeight := panelElementHeight(m.mainPanelHeight)
+	panCenter := panHeight / 2 // For making sure the cursor is at the center of the panel
 
-  if panlen == 0 {
-    return
-  }
+	if panlen == 0 {
+		return
+	}
 
-  if panHeight >= panlen {
-    panel.cursor = panlen - 1
-  } else {
-    if panel.cursor + panHeight >= panlen {
-      panel.cursor = panlen - 1
-      panel.render = panel.cursor - panCenter
-    } else {
-      panel.cursor += panHeight
-      panel.render = panel.cursor - panCenter
-    }
-  }
+	if panHeight >= panlen {
+		panel.cursor = panlen - 1
+	} else {
+		if panel.cursor+panHeight >= panlen {
+			panel.cursor = panlen - 1
+			panel.render = panel.cursor - panCenter
+		} else {
+			panel.cursor += panHeight
+			panel.render = panel.cursor - panCenter
+		}
+	}
 
 	m.fileModel.filePanels[m.filePanelFocusIndex] = panel
 }
@@ -403,135 +407,91 @@ func (m *model) itemSelectDown(wheel bool) {
 
 // ======================================== Sidebar controller ========================================
 
-// Yorukot: P.S God bless me, this sidebar controller code is really ugly...
+func (s *sidebarModel) controlListUp(wheel bool, mainPanelHeight int) {
 
-// Control sidebar panel list up
-func (m *model) controlSideBarListUp(wheel bool) {
+	// Todo : This snippet is duplicated everywhere. It can be better refractored outside
 	runTime := 1
 	if wheel {
 		runTime = wheelRunTime
 	}
-
 	for i := 0; i < runTime; i++ {
-		if m.sidebarModel.cursor > 0 {
-			m.sidebarModel.cursor--
-		} else {
-			m.sidebarModel.cursor = len(m.sidebarModel.directories) - 1
-		}
-		newDirectory := m.sidebarModel.directories[m.sidebarModel.cursor].location
-
-		for newDirectory == "Pinned+-*/=?" || newDirectory == "Disks+-*/=?" {
-			m.sidebarModel.cursor--
-			newDirectory = m.sidebarModel.directories[m.sidebarModel.cursor].location
-		}
-		changeToPlus := false
-		cursorRender := false
-		for !cursorRender {
-			totalHeight := 2
-			for i := m.sidebarModel.renderIndex; i < len(m.sidebarModel.directories); i++ {
-				if totalHeight >= m.mainPanelHeight {
-					break
-				}
-				directory := m.sidebarModel.directories[i]
-
-				if directory.location == "Pinned+-*/=?" {
-					totalHeight += 3
-					continue
-				}
-
-				if directory.location == "Disks+-*/=?" {
-					if m.mainPanelHeight-totalHeight <= 2 {
-						break
-					}
-					totalHeight += 3
-					continue
-				}
-
-				totalHeight++
-				if m.sidebarModel.cursor == i && m.focusPanel == sidebarFocus {
-					cursorRender = true
-				}
-			}
-
-			if changeToPlus {
-				m.sidebarModel.renderIndex++
-				continue
-			}
-
-			if !cursorRender {
-				m.sidebarModel.renderIndex--
-			}
-			if m.sidebarModel.renderIndex < 0 {
-				changeToPlus = true
-				m.sidebarModel.renderIndex++
-			}
-		}
-
-		if changeToPlus {
-			m.sidebarModel.renderIndex--
-		}
+		s.listUp(mainPanelHeight)
 	}
 }
 
-// Control sidebar panel list down
-func (m *model) controlSideBarListDown(wheel bool) {
+func (s *sidebarModel) listUp(mainPanelHeight int) {
+	slog.Debug("controlListUp called", "cursor", s.cursor,
+		"renderIndex", s.renderIndex, "directory count", len(s.directories))
+	if s.noActualDir() {
+		return
+	}
+	if s.cursor > 0 {
+		// Not at the top, can safely decrease
+		s.cursor--
+	} else {
+		// We are at the top. Move to the bottom
+		s.cursor = len(s.directories) - 1
+	}
+	// We should update even if cursor is at divider for now
+	// Otherwise dividers are sometimes skipped in render in case of
+	// large pinned directories
+	s.updateRenderIndex(mainPanelHeight)
+	if s.directories[s.cursor].isDivider() {
+		// cause another listUp trigger to move up.
+		s.listUp(mainPanelHeight)
+	}
+
+}
+
+func (s *sidebarModel) controlListDown(wheel bool, mainPanelHeight int) {
+
+	// Todo : This snippet is duplicated everywhere. It can be better refractored outside
 	runTime := 1
 	if wheel {
 		runTime = wheelRunTime
 	}
-
 	for i := 0; i < runTime; i++ {
-		lenDirs := len(m.sidebarModel.directories)
-		if m.sidebarModel.cursor < lenDirs-1 {
-			m.sidebarModel.cursor++
-		} else {
-			m.sidebarModel.cursor = 0
-		}
+		s.listDown(mainPanelHeight)
+	}
+}
 
-		newDirectory := m.sidebarModel.directories[m.sidebarModel.cursor].location
-		for newDirectory == "Pinned+-*/=?" || newDirectory == "Disks+-*/=?" {
-			m.sidebarModel.cursor++
-			if m.sidebarModel.cursor+1 > len(m.sidebarModel.directories) {
-				m.sidebarModel.cursor = 0
-			}
-			newDirectory = m.sidebarModel.directories[m.sidebarModel.cursor].location
-		}
-		cursorRender := false
-		for !cursorRender {
-			totalHeight := 2
-			for i := m.sidebarModel.renderIndex; i < len(m.sidebarModel.directories); i++ {
-				if totalHeight >= m.mainPanelHeight {
-					break
-				}
+func (s *sidebarModel) listDown(mainPanelHeight int) {
+	slog.Debug("controlListDown called", "cursor", s.cursor,
+		"renderIndex", s.renderIndex, "directory count", len(s.directories))
+	if s.noActualDir() {
+		return
+	}
+	if s.cursor < len(s.directories)-1 {
+		// Not at the bottom, can safely increase
+		s.cursor++
+	} else {
+		// We are at the bottom. Move to the top
+		s.cursor = 0
+	}
 
-				directory := m.sidebarModel.directories[i]
+	// We should update even if cursor is at divider for now
+	// Otherwise dividers are sometimes skipped in render in case of
+	// large pinned directories
+	s.updateRenderIndex(mainPanelHeight)
 
-				if directory.location == "Pinned+-*/=?" {
-					totalHeight += 3
-					continue
-				}
+	// Move below special divider directories
+	if s.directories[s.cursor].isDivider() {
+		// cause another listDown trigger to move down.
+		s.listDown(mainPanelHeight)
+	}
+}
 
-				if directory.location == "Disks+-*/=?" {
-					if m.mainPanelHeight-totalHeight <= 2 {
-						break
-					}
-					totalHeight += 3
-					continue
-				}
+func (m *model) sidebarSearchBarFocus() {
 
-				totalHeight++
-				if m.sidebarModel.cursor == i && m.focusPanel == sidebarFocus {
-					cursorRender = true
-				}
-			}
-
-			if !cursorRender {
-				m.sidebarModel.renderIndex++
-			}
-			if m.sidebarModel.renderIndex > m.sidebarModel.cursor {
-				m.sidebarModel.renderIndex = 0
-			}
-		}
+	if m.sidebarModel.searchBar.Focused() {
+		// Ideally Code should never reach here. Once sidebar is focussed, we should
+		// not cause sidebarSearchBarFocus() event by pressing search key
+		// Should we use Runtime panic asserts ?
+		slog.Error("sidebarSearchBarFocus() called on Focussed sidebar")
+		m.sidebarModel.searchBar.Blur()
+	} else {
+		m.sidebarModel.searchBar.Focus()
+		m.firstTextInput = true
 	}
 }
 
