@@ -11,16 +11,17 @@ func containsKey(v string, a []string) string {
 	return ""
 }
 
-// mainKey handles most of key commands in the regular state of the application. For 
+// mainKey handles most of key commands in the regular state of the application. For
 // keys that performs actions in multiple panels, like going up or down,
 // check the state of model m and handle properly.
-func (m *model) mainKey(msg string, cmd tea.Cmd) ( tea.Cmd) {
+func (m *model) mainKey(msg string, cmd tea.Cmd) tea.Cmd {
+
 	switch msg {
 
-    // If move up Key is pressed, check the current state and executes
+	// If move up Key is pressed, check the current state and executes
 	case containsKey(msg, hotkeys.ListUp):
 		if m.focusPanel == sidebarFocus {
-			m.controlSideBarListUp(false)
+			m.sidebarModel.controlListUp(false, m.mainPanelHeight)
 		} else if m.focusPanel == processBarFocus {
 			m.controlProcessbarListUp(false)
 		} else if m.focusPanel == metadataFocus {
@@ -33,10 +34,10 @@ func (m *model) mainKey(msg string, cmd tea.Cmd) ( tea.Cmd) {
 			}()
 		}
 
-    // If move down Key is pressed, check the current state and executes
+		// If move down Key is pressed, check the current state and executes
 	case containsKey(msg, hotkeys.ListDown):
 		if m.focusPanel == sidebarFocus {
-			m.controlSideBarListDown(false)
+			m.sidebarModel.controlListDown(false, m.mainPanelHeight)
 		} else if m.focusPanel == processBarFocus {
 			m.controlProcessbarListDown(false)
 		} else if m.focusPanel == metadataFocus {
@@ -49,11 +50,11 @@ func (m *model) mainKey(msg string, cmd tea.Cmd) ( tea.Cmd) {
 			}()
 		}
 
-  case containsKey(msg, hotkeys.PageUp):
-    m.controlFilePanelPgUp()
+	case containsKey(msg, hotkeys.PageUp):
+		m.controlFilePanelPgUp()
 
-  case containsKey(msg, hotkeys.PageDown):
-    m.controlFilePanelPgDown()
+	case containsKey(msg, hotkeys.PageDown):
+		m.controlFilePanelPgDown()
 
 	case containsKey(msg, hotkeys.ChangePanelMode):
 		m.changeFilePanelMode()
@@ -142,6 +143,12 @@ func (m *model) normalAndBrowserModeKey(msg string) {
 		if m.focusPanel == sidebarFocus && (msg == containsKey(msg, hotkeys.Confirm)) {
 			m.sidebarSelectDirectory()
 		}
+		if m.focusPanel == sidebarFocus && (msg == containsKey(msg, hotkeys.FilePanelItemRename)) {
+			m.pinnedItemRename()
+		}
+		if m.focusPanel == sidebarFocus && (msg == containsKey(msg, hotkeys.SearchBar)) {
+			m.sidebarSearchBarFocus()
+		}
 		return
 	}
 	// Check if in the select mode and focusOn filepanel
@@ -158,9 +165,9 @@ func (m *model) normalAndBrowserModeKey(msg string) {
 				m.deleteItemWarn()
 			}()
 		case containsKey(msg, hotkeys.CopyItems):
-			m.copyMultipleItem()
+			m.copyMultipleItem(false)
 		case containsKey(msg, hotkeys.CutItems):
-			m.cutMultipleItem()
+			m.copyMultipleItem(true)
 		case containsKey(msg, hotkeys.FilePanelSelectAllItem):
 			m.selectAllItem()
 		}
@@ -177,21 +184,21 @@ func (m *model) normalAndBrowserModeKey(msg string) {
 			m.deleteItemWarn()
 		}()
 	case containsKey(msg, hotkeys.CopyItems):
-		m.copySingleItem()
+		m.copySingleItem(false)
 	case containsKey(msg, hotkeys.CutItems):
-		m.cutSingleItem()
+		m.copySingleItem(true)
 	case containsKey(msg, hotkeys.FilePanelItemRename):
 		m.panelItemRename()
 	case containsKey(msg, hotkeys.SearchBar):
 		m.searchBarFocus()
 	case containsKey(msg, hotkeys.CopyPath):
 		m.copyPath()
-  case containsKey(msg, hotkeys.CopyPWD):
-    m.copyPWD()
+	case containsKey(msg, hotkeys.CopyPWD):
+		m.copyPWD()
 	}
 }
 
-// Check the hotkey to cancel operation or create file 
+// Check the hotkey to cancel operation or create file
 func (m *model) typingModalOpenKey(msg string) {
 	switch msg {
 	case containsKey(msg, hotkeys.CancelTyping):
@@ -238,7 +245,7 @@ func (m *model) warnModalOpenKey(msg string) {
 
 			}
 		case confirmRenameItem:
-            m.confirmRename()
+			m.confirmRename()
 		}
 	}
 }
@@ -262,16 +269,14 @@ func (m *model) sortOptionsKey(msg string) {
 	switch msg {
 	case containsKey(msg, hotkeys.OpenSortOptionsMenu):
 		m.cancelSortOptions()
-	case containsKey(msg, hotkeys.CancelTyping):
+	case containsKey(msg, hotkeys.Quit):
 		m.cancelSortOptions()
-	case containsKey(msg, hotkeys.ConfirmTyping):
+	case containsKey(msg, hotkeys.Confirm):
 		m.confirmSortOptions()
 	case containsKey(msg, hotkeys.ListUp):
 		m.sortOptionsListUp()
 	case containsKey(msg, hotkeys.ListDown):
 		m.sortOptionsListDown()
-	case containsKey(msg, hotkeys.Quit):
-		m.cancelSortOptions()
 	}
 }
 
@@ -288,6 +293,26 @@ func (m *model) renamingKey(msg string) {
 	}
 }
 
+func (m *model) sidebarRenamingKey(msg string) {
+	switch msg {
+	case containsKey(msg, hotkeys.CancelTyping):
+		m.cancelSidebarRename()
+	case containsKey(msg, hotkeys.ConfirmTyping):
+		m.confirmSidebarRename()
+	}
+}
+
+func (m *model) sidebarSearchBarKey(msg string) {
+	switch msg {
+	case containsKey(msg, hotkeys.CancelTyping):
+		m.sidebarModel.searchBar.Blur()
+		m.sidebarModel.searchBar.SetValue("")
+	case containsKey(msg, hotkeys.ConfirmTyping):
+		m.sidebarModel.searchBar.Blur()
+		m.sidebarModel.resetCursor()
+	}
+}
+
 // Check the key input and cancel or confirms the search
 func (m *model) focusOnSearchbarKey(msg string) {
 	switch msg {
