@@ -1,7 +1,7 @@
 package internal
 
 import (
-	"log"
+	"log/slog"
 	"os"
 	"path/filepath"
 	"strings"
@@ -25,7 +25,6 @@ var theme ThemeType
 var Config ConfigType
 var hotkeys HotkeysType
 
-var logOutput *os.File
 var et *exiftool.Exiftool
 
 var channel = make(chan channelMessage, 1000)
@@ -52,21 +51,39 @@ func (m model) Init() tea.Cmd {
 // Update function for bubble tea to provide internal communication to the
 // application
 func (m model) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
+	slog.Debug("model.Update() called")
 	var cmd tea.Cmd
 
+	m.updateSidebarState(msg, &cmd)
+
 	switch msg := msg.(type) {
 	case channelMessage:
 		m.handleChannelMessage(msg)
 	case tea.WindowSizeMsg:
 		m.handleWindowResize(msg)
 	case tea.MouseMsg:
-		m, cmd = wheelMainAction(msg.String(), m, cmd)
+		cmd = wheelMainAction(msg.String(), &m, cmd)
 	case tea.KeyMsg:
-		m, cmd = m.handleKeyInput(msg, cmd)
+		cmd = m.handleKeyInput(msg, cmd)
 	}
 
 	m.updateFilePanelsState(msg, &cmd)
-	m.sidebarModel.directories = getDirectories()
+
+	if m.sidebarModel.searchBar.Value() != "" {
+		// Todo : All updates of sideBar must be moved to seperate struct functions
+		// we have to keep the state of sidebar consistent, and keep values of
+		// cursor, directories, renderIndex sane for each update, and it has to
+		// take care at one single place, not everywhere we use sideBar
+		m.sidebarModel.directories = getFilteredDirectories(m.sidebarModel.searchBar.Value())
+		if m.sidebarModel.isCursorInvalid() {
+			m.sidebarModel.resetCursor()
+		}
+	} else {
+		m.sidebarModel.directories = getDirectories()
+		if m.sidebarModel.isCursorInvalid() {
+			m.sidebarModel.resetCursor()
+		}
+	}
 
 	// check if there already have listening message
 	if !ListeningMessage {
@@ -74,11 +91,13 @@ func (m model) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
 	}
 
 	m.getFilePanelItems()
-
+	if !m.firstLoadingComplete {
+		m.firstLoadingComplete = true
+	}
 	return m, tea.Batch(cmd)
 }
 
-// Handle message exchanging whithin the application
+// Handle message exchanging within the application
 func (m *model) handleChannelMessage(msg channelMessage) {
 	switch msg.messageType {
 	case sendWarnModal:
@@ -117,7 +136,7 @@ func (m *model) setFilePreviewWidth(width int) {
 	if Config.FilePreviewWidth == 0 {
 		m.fileModel.filePreview.width = (width - Config.SidebarWidth - (4 + (len(m.fileModel.filePanels))*2)) / (len(m.fileModel.filePanels) + 1)
 	} else if Config.FilePreviewWidth > 10 || Config.FilePreviewWidth == 1 {
-		log.Fatalln("Config file file_preview_width invalidation")
+		LogAndExit("Config file file_preview_width invalidation")
 	} else {
 		m.fileModel.filePreview.width = (width - Config.SidebarWidth) / Config.FilePreviewWidth
 	}
@@ -176,10 +195,27 @@ func (m *model) setHelpMenuSize() {
 
 // Identify the current state of the application m and properly handle the
 // msg keybind pressed
-func (m model) handleKeyInput(msg tea.KeyMsg, cmd tea.Cmd) (model, tea.Cmd) {
+func (m *model) handleKeyInput(msg tea.KeyMsg, cmd tea.Cmd) tea.Cmd {
+
+	slog.Debug("model.handleKeyInput", "msg", msg, "typestr", msg.Type.String(),
+		"runes", msg.Runes, "type", int(msg.Type), "paste", msg.Paste,
+		"alt", msg.Alt)
+	slog.Debug("model.handleKeyInput. model info. ",
+		"filePanelFocusIndex", m.filePanelFocusIndex,
+		"filePanel.focusType", m.fileModel.filePanels[m.filePanelFocusIndex].focusType,
+		"filePanel.panelMode", m.fileModel.filePanels[m.filePanelFocusIndex].panelMode,
+		"typingModal.open", m.typingModal.open,
+		"warnModal.open", m.warnModal.open,
+		"fileModel.renaming", m.fileModel.renaming,
+		"searchBar.focussed", m.fileModel.filePanels[m.filePanelFocusIndex].searchBar.Focused(),
+		"helpMenu.open", m.helpMenu.open,
+		"firstTextInput", m.firstTextInput,
+		"focusPanel", m.focusPanel,
+	)
+
 	if firstUse {
 		firstUse = false
-		return m, cmd
+		return cmd
 	}
 
 	if m.typingModal.open {
@@ -189,10 +225,15 @@ func (m model) handleKeyInput(msg tea.KeyMsg, cmd tea.Cmd) (model, tea.Cmd) {
 		// If renaming a object
 	} else if m.fileModel.renaming {
 		m.renamingKey(msg.String())
+	} else if m.sidebarModel.renaming {
+		m.sidebarRenamingKey(msg.String())
 		// If search bar is open
 	} else if m.fileModel.filePanels[m.filePanelFocusIndex].searchBar.Focused() {
 		m.focusOnSearchbarKey(msg.String())
 		// If sort options menu is open
+	} else if m.sidebarModel.searchBar.Focused() {
+		m.sidebarSearchBarKey(msg.String())
+		// If sort options menu is open
 	} else if m.fileModel.filePanels[m.filePanelFocusIndex].sortOptions.open {
 		m.sortOptionsKey(msg.String())
 		// If help menu is open
@@ -206,23 +247,23 @@ func (m model) handleKeyInput(msg tea.KeyMsg, cmd tea.Cmd) (model, tea.Cmd) {
 		quit := m.confirmToQuitSuperfile(msg.String())
 		if quit {
 			m.quitSuperfile()
-			return m, tea.Quit
+			return tea.Quit
 		}
-		// If quiting input pressed, check if has any runing process and displays a
+		// If quiting input pressed, check if has any running process and displays a
 		// warn. Otherwise just quits application
 	} else if msg.String() == containsKey(msg.String(), hotkeys.Quit) {
 		if m.hasRunningProcesses() {
 			m.warnModalForQuit()
-			return m, cmd
+			return cmd
 		}
 
 		m.quitSuperfile()
-		return m, tea.Quit
+		return tea.Quit
 	} else {
 		// Handles general kinds of inputs in the regular state of the application
 		cmd = m.mainKey(msg.String(), cmd)
 	}
-	return m, cmd
+	return cmd
 }
 
 // Update the file panel state. Change name of renamed files, filter out files
@@ -235,12 +276,6 @@ func (m *model) updateFilePanelsState(msg tea.Msg, cmd *tea.Cmd) {
 		focusPanel.rename, *cmd = focusPanel.rename.Update(msg)
 	} else if focusPanel.searchBar.Focused() {
 		focusPanel.searchBar, *cmd = focusPanel.searchBar.Update(msg)
-		for _, hotkey := range hotkeys.SearchBar {
-			if hotkey == focusPanel.searchBar.Value() {
-				focusPanel.searchBar.SetValue("")
-				break
-			}
-		}
 	} else if m.commandLine.input.Focused() {
 		m.commandLine.input, *cmd = m.commandLine.input.Update(msg)
 	} else if m.typingModal.open {
@@ -252,6 +287,20 @@ func (m *model) updateFilePanelsState(msg tea.Msg, cmd *tea.Cmd) {
 	}
 }
 
+// Update the sidebar state. Change name of the renaming pinned directory.
+func (m *model) updateSidebarState(msg tea.Msg, cmd *tea.Cmd) {
+	sidebar := &m.sidebarModel
+	if sidebar.renaming {
+		sidebar.rename, *cmd = sidebar.rename.Update(msg)
+	} else if sidebar.searchBar.Focused() {
+		sidebar.searchBar, *cmd = sidebar.searchBar.Update(msg)
+	}
+
+	if sidebar.cursor < 0 {
+		sidebar.cursor = 0
+	}
+}
+
 // Check if there's any processes running in background
 func (m *model) hasRunningProcesses() bool {
 	for _, data := range m.processBarModel.process {
@@ -271,6 +320,10 @@ func (m *model) warnModalForQuit() {
 
 // Implement View function for bubble tea model to handle visualization.
 func (m model) View() string {
+	slog.Debug("model.View() called")
+	if !m.firstLoadingComplete {
+		return "Loading..."
+	}
 	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
 	// check is the terminal size enough
 	if m.fullHeight < minimumHeight || m.fullWidth < minimumWidth {
@@ -385,9 +438,9 @@ func (m *model) getFilePanelItems() {
 		nowTime := time.Now()
 		// Check last time each element was updated, if less then 3 seconds ignore
 		if filePanel.focusType == noneFocus && nowTime.Sub(filePanel.lastTimeGetElement) < 3*time.Second {
-      if !m.updatedToggleDotFile {
-        continue
-      }
+			if !m.updatedToggleDotFile {
+				continue
+			}
 		}
 
 		focusPanelReRender := false
@@ -408,9 +461,9 @@ func (m *model) getFilePanelItems() {
 
 		// Get file names based on search bar filter
 		if filePanel.searchBar.Value() != "" {
-			fileElement = returnFolderElementBySearchString(filePanel.location, m.toggleDotFile, filePanel.searchBar.Value())
+			fileElement = returnDirElementBySearchString(filePanel.location, m.toggleDotFile, filePanel.searchBar.Value())
 		} else {
-			fileElement = returnFolderElement(filePanel.location, m.toggleDotFile, filePanel.sortOptions.data)
+			fileElement = returnDirElement(filePanel.location, m.toggleDotFile, filePanel.sortOptions.data)
 		}
 		// Update file panel list
 		filePanel.element = fileElement
@@ -418,23 +471,27 @@ func (m *model) getFilePanelItems() {
 		m.fileModel.filePanels[i].lastTimeGetElement = nowTime
 	}
 
-  m.updatedToggleDotFile = false
+	m.updatedToggleDotFile = false
 }
 
 // Close superfile application. Cd into the curent dir if CdOnQuit on and save
 // the path in state direcotory
-func (m model) quitSuperfile() {
-    // close exiftool session
-    if Config.Metadata {
-        et.Close();
-    }
-    // cd on quit
-    currentDir := m.fileModel.filePanels[m.filePanelFocusIndex].location
-    variable.LastDir = currentDir
+func (m *model) quitSuperfile() {
+	// close exiftool session
+	if Config.Metadata && et != nil {
+		et.Close()
+	}
+	// cd on quit
+	currentDir := m.fileModel.filePanels[m.filePanelFocusIndex].location
+	variable.LastDir = currentDir
 
-    if Config.CdOnQuit {
-        // escape single quote
-        currentDir = strings.ReplaceAll(currentDir, "'", "'\\''")
-        os.WriteFile(variable.SuperFileStateDir+"/lastdir", []byte("cd '"+currentDir+"'"), 0755)
-    }
+	if Config.CdOnQuit {
+		// escape single quote
+		currentDir = strings.ReplaceAll(currentDir, "'", "'\\''")
+		err := os.WriteFile(variable.LastDirFile, []byte("cd '"+currentDir+"'"), 0755)
+		if err != nil {
+			slog.Error("Error during writing lastdir file", "error", err)
+		}
+	}
+	slog.Debug("Quitting superfile", "current dir", currentDir)
 }
