@@ -4,13 +4,14 @@ import (
 	"bufio"
 	"fmt"
 	"image"
+	"log/slog"
 	"os"
 	"path/filepath"
 	"sort"
 	"strconv"
 	"strings"
 
-	"github.com/alecthomas/chroma/lexers"
+	"github.com/alecthomas/chroma/v2/lexers"
 	"github.com/charmbracelet/lipgloss"
 	"github.com/charmbracelet/x/exp/term/ansi"
 	"github.com/yorukot/ansichroma"
@@ -18,62 +19,78 @@ import (
 	filepreview "github.com/yorukot/superfile/src/pkg/file_preview"
 )
 
-func (m model) sidebarRender() string {
+func (m *model) sidebarRender() string {
 	if Config.SidebarWidth == 0 {
 		return ""
 	}
-	superfileTitle := sidebarTitleStyle.Render("    " + icon.SuperfileIcon + " superfile")
-	superfileTitle = ansi.Truncate(superfileTitle, Config.SidebarWidth, "")
-	s := superfileTitle
-	s += "\n"
+	slog.Debug("Rendering sidebar.", "cursor", m.sidebarModel.cursor,
+		"renderIndex", m.sidebarModel.renderIndex, "dirs count", len(m.sidebarModel.directories),
+		"sidebar focused", m.focusPanel == sidebarFocus)
 
-	pinnedDivider := "\n" + sidebarTitleStyle.Render("󰐃 Pinned") + sidebarDividerStyle.Render(" ───────────") + "\n"
-	disksDivider := "\n" + sidebarTitleStyle.Render("󱇰 Disks") + sidebarDividerStyle.Render(" ────────────") + "\n"
-	disksDivider = ansi.Truncate(disksDivider, Config.SidebarWidth, "")
-	pinnedDivider = ansi.Truncate(pinnedDivider, Config.SidebarWidth, "")
+	s := sideBarSuperfileTitle + "\n"
 
-	totalHeight := 2
-	for i := m.sidebarModel.renderIndex; i < len(m.sidebarModel.directories); i++ {
-		if totalHeight >= m.mainPanelHeight {
-			break
-		} else {
-			s += "\n"
-		}
-
-		directory := m.sidebarModel.directories[i]
-
-		if directory.location == "Pinned+-*/=?" {
-			s += pinnedDivider
-			totalHeight += 3
-			continue
-		}
-
-		if directory.location == "Disks+-*/=?" {
-			if m.mainPanelHeight-totalHeight <= 2 {
-				break
-			}
-			s += disksDivider
-			totalHeight += 3
-			continue
-		}
-
-		totalHeight++
-		cursor := " "
-		if m.sidebarModel.cursor == i && m.focusPanel == sidebarFocus {
-			cursor = icon.Cursor
-		}
-
-		if directory.location == m.fileModel.filePanels[m.filePanelFocusIndex].location {
-			s += filePanelCursorStyle.Render(cursor+" ") + sidebarSelectedStyle.Render(truncateText(directory.name, Config.SidebarWidth-2, "..."))
-		} else {
-			s += filePanelCursorStyle.Render(cursor+" ") + sidebarStyle.Render(truncateText(directory.name, Config.SidebarWidth-2, "..."))
-		}
+	if m.sidebarModel.searchBar.Focused() || m.sidebarModel.searchBar.Value() != "" || m.focusPanel == sidebarFocus {
+		m.sidebarModel.searchBar.Placeholder = "(" + hotkeys.SearchBar[0] + ")" + " Search"
+		s += "\n" + ansi.Truncate(m.sidebarModel.searchBar.View(), Config.SidebarWidth-2, "...")
 	}
 
+	if m.sidebarModel.noActualDir() {
+		s += "\n" + sideBarNoneText
+		return sideBarBorderStyle(m.mainPanelHeight, m.focusPanel).Render(s)
+	}
+
+	s += m.sidebarModel.directoriesRender(m.mainPanelHeight,
+		m.fileModel.filePanels[m.filePanelFocusIndex].location, m.focusPanel == sidebarFocus)
+
 	return sideBarBorderStyle(m.mainPanelHeight, m.focusPanel).Render(s)
 }
 
-func (m model) filePanelRender() string {
+func (s *sidebarModel) directoriesRender(mainPanelHeight int, curFilePanelFileLocation string, sideBarFocussed bool) string {
+
+	// Cursor should always point to a valid directory at this point
+	if s.isCursorInvalid() {
+		slog.Error("Unexpected situation in sideBar Model. "+
+			"Cursor is at invalid postion, while there are valide directories", "cursor", s.cursor,
+			"directory count", len(s.directories))
+		return ""
+	}
+
+	res := ""
+	totalHeight := sideBarInitialHeight
+	for i := s.renderIndex; i < len(s.directories); i++ {
+		if totalHeight+s.directories[i].requiredHeight() > mainPanelHeight {
+			break
+		}
+		res += "\n"
+
+		totalHeight += s.directories[i].requiredHeight()
+
+		if s.directories[i] == pinnedDividerDir {
+			res += "\n" + sideBarPinnedDivider
+		} else if s.directories[i] == diskDividerDir {
+			res += "\n" + sideBarDisksDivider
+		} else {
+			cursor := " "
+			if s.cursor == i && sideBarFocussed && !s.searchBar.Focused() {
+				cursor = icon.Cursor
+			}
+			if s.renaming && s.cursor == i {
+				res += s.rename.View()
+			} else {
+				renderStyle := sidebarStyle
+				if s.directories[i].location == curFilePanelFileLocation {
+					renderStyle = sidebarSelectedStyle
+				}
+				res += filePanelCursorStyle.Render(cursor+" ") +
+					renderStyle.Render(truncateText(s.directories[i].name, Config.SidebarWidth-2, "..."))
+			}
+		}
+	}
+	return res
+}
+
+// This also modifies the m.fileModel.filePanels
+func (m *model) filePanelRender() string {
 	// file panel
 	f := make([]string, 10)
 	for i, filePanel := range m.fileModel.filePanels {
@@ -170,8 +187,8 @@ func (m model) filePanelRender() string {
 				if filePanel.renaming && h == filePanel.cursor {
 					f[i] += filePanel.rename.View() + endl
 				} else {
-					_, err := os.ReadDir(filePanel.element[h].location);
-					f[i] += filePanelCursorStyle.Render(cursor+" ") + prettierName(filePanel.element[h].name, m.fileModel.width-5, filePanel.element[h].directory||(err == nil), isItemSelected, filePanelBGColor) + endl
+					_, err := os.ReadDir(filePanel.element[h].location)
+					f[i] += filePanelCursorStyle.Render(cursor+" ") + prettierName(filePanel.element[h].name, m.fileModel.width-5, filePanel.element[h].directory || (err == nil), isItemSelected, filePanelBGColor) + endl
 				}
 			}
 			cursorPosition := strconv.Itoa(filePanel.cursor + 1)
@@ -189,7 +206,7 @@ func (m model) filePanelRender() string {
 	}
 	return filePanelRender
 }
-func (m model) processBarRender() string {
+func (m *model) processBarRender() string {
 	// save process in the array
 	var processes []process
 	for _, p := range m.processBarModel.process {
@@ -274,7 +291,8 @@ func (m model) processBarRender() string {
 	return processRender
 }
 
-func (m model) metadataRender() string {
+// This updates m.fileMetaData
+func (m *model) metadataRender() string {
 	// process bar
 	metaDataBar := ""
 	if len(m.fileMetaData.metaData) == 0 && len(m.fileModel.filePanels[m.filePanelFocusIndex].element) > 0 && !m.fileModel.renaming {
@@ -330,7 +348,7 @@ func (m model) metadataRender() string {
 	return metaDataBar
 }
 
-func (m model) clipboardRender() string {
+func (m *model) clipboardRender() string {
 
 	// render
 	clipboardRender := ""
@@ -343,7 +361,7 @@ func (m model) clipboardRender() string {
 			} else {
 				fileInfo, err := os.Stat(m.copyItems.items[i])
 				if err != nil {
-					outPutLog("Clipboard render function get item state error", err)
+					slog.Error("Clipboard render function get item state ", "error", err)
 				}
 				if !os.IsNotExist(err) {
 					clipboardRender += clipboardPrettierName(m.copyItems.items[i], footerWidth(m.fullWidth)-3, fileInfo.IsDir(), false) + "\n"
@@ -366,7 +384,7 @@ func (m model) clipboardRender() string {
 	return clipboardRender
 }
 
-func (m model) terminalSizeWarnRender() string {
+func (m *model) terminalSizeWarnRender() string {
 	fullWidthString := strconv.Itoa(m.fullWidth)
 	fullHeightString := strconv.Itoa(m.fullHeight)
 	minimumWidthString := strconv.Itoa(minimumWidth)
@@ -390,7 +408,7 @@ func (m model) terminalSizeWarnRender() string {
 		heightString + terminalCorrectSize.Render(minimumHeightString))
 }
 
-func (m model) terminalSizeWarnAfterFirstRender() string {
+func (m *model) terminalSizeWarnAfterFirstRender() string {
 	minimumWidthInt := Config.SidebarWidth + 20*len(m.fileModel.filePanels) + 20 - 1
 	minimumWidthString := strconv.Itoa(minimumWidthInt)
 	fullWidthString := strconv.Itoa(m.fullWidth)
@@ -416,8 +434,8 @@ func (m model) terminalSizeWarnAfterFirstRender() string {
 		heightString + terminalCorrectSize.Render(minimumHeightString))
 }
 
-func (m model) typineModalRender() string {
-	previewPath := m.typingModal.location + "/" + m.typingModal.textInput.Value()
+func (m *model) typineModalRender() string {
+	previewPath := filepath.Join(m.typingModal.location, m.typingModal.textInput.Value())
 
 	fileLocation := filePanelTopDirectoryIconStyle.Render(" "+icon.Directory+icon.Space) +
 		filePanelTopPathStyle.Render(truncateTextBeginning(previewPath, modalWidth-4, "...")) + "\n"
@@ -432,7 +450,7 @@ func (m model) typineModalRender() string {
 	return modalBorderStyle(modalHeight, modalWidth).Render(fileLocation + "\n" + m.typingModal.textInput.View() + "\n\n" + tip)
 }
 
-func (m model) introduceModalRender() string {
+func (m *model) introduceModalRender() string {
 	title := sidebarTitleStyle.Render(" Thanks for using superfile!!") + modalStyle.Render("\n You can read the following information before starting to use it!")
 	vimUserWarn := processErrorStyle.Render("  ** Very importantly ** If you are a Vim/Nvim user, go to:\n  https://superfile.netlify.app/configure/custom-hotkeys/ to change your hotkey settings!")
 	subOne := sidebarTitleStyle.Render("  (1)") + modalStyle.Render(" If this is your first time, make sure you read:\n      https://superfile.netlify.app/getting-started/tutorial/")
@@ -442,7 +460,7 @@ func (m model) introduceModalRender() string {
 	return firstUseModal(m.helpMenu.height, m.helpMenu.width).Render(title + "\n\n" + vimUserWarn + "\n\n" + subOne + "\n\n" + subTwo + "\n\n" + subThree + "\n\n" + subFour + "\n\n")
 }
 
-func (m model) warnModalRender() string {
+func (m *model) warnModalRender() string {
 	title := m.warnModal.title
 	content := m.warnModal.content
 	confirm := modalConfirm.Render(" (" + hotkeys.Confirm[0] + ") Confirm ")
@@ -451,7 +469,7 @@ func (m model) warnModalRender() string {
 	return modalBorderStyle(modalHeight, modalWidth).Render(title + "\n\n" + content + "\n\n" + tip)
 }
 
-func (m model) helpMenuRender() string {
+func (m *model) helpMenuRender() string {
 	helpMenuContent := ""
 	maxKeyLength := 0
 
@@ -536,7 +554,7 @@ func (m model) helpMenuRender() string {
 	return helpMenuModalBorderStyle(m.helpMenu.height, m.helpMenu.width, bottomBorder).Render(helpMenuContent)
 }
 
-func (m model) sortOptionsRender() string {
+func (m *model) sortOptionsRender() string {
 	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
 	sortOptionsContent := modalTitleStyle.Render(" Sort Options") + "\n\n"
 	for i, option := range panel.sortOptions.data.options {
@@ -551,7 +569,36 @@ func (m model) sortOptionsRender() string {
 	return sortOptionsModalBorderStyle(panel.sortOptions.height, panel.sortOptions.width, bottomBorder).Render(sortOptionsContent)
 }
 
-func (m model) filePreviewPanelRender() string {
+func readFileContent(filepath string, maxLineLength int, previewLine int) (string, error) {
+	// String builder is much better for efficiency
+	// See - https://stackoverflow.com/questions/1760757/how-to-efficiently-concatenate-strings-in-go/47798475#47798475
+	var resultBuilder strings.Builder
+	file, err := os.Open(filepath)
+	if err != nil {
+		return resultBuilder.String(), err
+	}
+	defer file.Close()
+
+	scanner := bufio.NewScanner(file)
+	lineCount := 0
+	for scanner.Scan() {
+		line := scanner.Text()
+		if len(line) > maxLineLength {
+			line = line[:maxLineLength]
+		}
+		// This is critical to avoid layout break, removes non Printable ASCII control characters.
+		line = makePrintable(line)
+		resultBuilder.WriteString(line + "\n")
+		lineCount++
+		if previewLine > 0 && lineCount >= previewLine {
+			break
+		}
+	}
+	// returns the first non-EOF error that was encountered by the [Scanner]
+	return resultBuilder.String(), scanner.Err()
+}
+
+func (m *model) filePreviewPanelRender() string {
 	previewLine := m.mainPanelHeight + 2
 	m.fileModel.filePreview.width += m.fullWidth - Config.SidebarWidth - m.fileModel.filePreview.width - ((m.fileModel.width + 2) * len(m.fileModel.filePanels)) - 2
 
@@ -567,12 +614,12 @@ func (m model) filePreviewPanelRender() string {
 	fileInfo, err := os.Stat(itemPath)
 
 	if err != nil {
-		outPutLog("error get file info", err)
+		slog.Error("Error get file info", "error", err)
 		return box.Render("\n --- " + icon.Error + " Error get file info ---")
 	}
 
 	ext := filepath.Ext(itemPath)
-	// check if the file is unsipported file, cuz pdf will cause error
+	// check if the file is unsupported file, cuz pdf will cause error
 	if ext == ".pdf" || ext == ".torrent" {
 		return box.Render("\n --- " + icon.Error + " Unsupported formats ---")
 	}
@@ -583,7 +630,7 @@ func (m model) filePreviewPanelRender() string {
 
 		files, err := os.ReadDir(dirPath)
 		if err != nil {
-			outPutLog("Error render directory preview", err)
+			slog.Error("Error render directory preview", "error", err)
 			return box.Render("\n --- " + icon.Error + " Error render directory preview ---")
 		}
 
@@ -614,15 +661,15 @@ func (m model) filePreviewPanelRender() string {
 
 	if isImageFile(itemPath) {
 		if !m.fileModel.filePreview.open {
-            return box.Render("\n --- Preview panel is closed ---")
-        }
+			return box.Render("\n --- Preview panel is closed ---")
+		}
 		ansiRender, err := filepreview.ImagePreview(itemPath, m.fileModel.filePreview.width, previewLine, theme.FilePanelBG)
 		if err == image.ErrFormat {
 			return box.Render("\n --- " + icon.Error + " Unsupported image formats ---")
 		}
 
 		if err != nil {
-			outPutLog("Error covernt image to ansi", err)
+			slog.Error("Error covernt image to ansi", "error", err)
 			return box.Render("\n --- " + icon.Error + " Error covernt image to ansi ---")
 		}
 
@@ -630,88 +677,44 @@ func (m model) filePreviewPanelRender() string {
 	}
 
 	format := lexers.Match(filepath.Base(itemPath))
-	if format != nil {
-		var codeHighlight string
-		var err error
-		var fileContent string
-		file, err := os.Open(itemPath)
+
+	if format == nil {
+		isText, err := isTextFile(itemPath)
 		if err != nil {
-			outPutLog(err)
-			return box.Render("\n --- " + icon.Error + " Error open file ---")
-		}
-		defer file.Close()
-
-		scanner := bufio.NewScanner(file)
-		lineCount := 0
-
-		maxLineLength := m.fileModel.width + 20
-		for scanner.Scan() {
-			line := scanner.Text()
-			if len(line) > maxLineLength {
-				line = line[:maxLineLength]
-			}
-			fileContent += line + "\n"
-			lineCount++
-			if previewLine > 0 && lineCount >= previewLine {
-				break
-			}
-		}
-
-		if Config.TransparentBackground {
-			codeHighlight, err = ansichroma.HightlightString(fileContent, format.Config().Name, theme.CodeSyntaxHighlightTheme, "")
-		} else {
-			codeHighlight, err = ansichroma.HightlightString(fileContent, format.Config().Name, theme.CodeSyntaxHighlightTheme, theme.FilePanelBG)
-		}
-		if err != nil {
-			outPutLog("Error render code highlight", err)
-			return box.Render("\n --- " + icon.Error + " Error render code highlight ---")
-		}
-		if codeHighlight == "" {
-			return box.Render("\n --- empty ---")
-		}
-
-		codeHighlight = checkAndTruncateLineLengths(codeHighlight, m.fileModel.filePreview.width)
-
-		return box.Render(codeHighlight)
-	} else {
-		textFile, err := isTextFile(itemPath)
-		if err != nil {
-			outPutLog("Error check text file", err)
-		}
-		if textFile {
-			var fileContent string
-			file, err := os.Open(itemPath)
-			if err != nil {
-				outPutLog(err)
-				return box.Render("\n --- " + icon.Error + " Error open file ---")
-			}
-			defer file.Close()
-
-			scanner := bufio.NewScanner(file)
-			lineCount := 0
-
-			for scanner.Scan() {
-				fileContent += scanner.Text() + "\n"
-				lineCount++
-				if previewLine > 0 && lineCount >= previewLine {
-					break
-				}
-			}
-
-			if err := scanner.Err(); err != nil {
-				outPutLog(err)
-				return box.Render("\n --- " + icon.Error + " Error open file ---")
-			}
-
-			textContent := checkAndTruncateLineLengths(fileContent, m.fileModel.filePreview.width)
-
-			return box.Render(textContent)
+			slog.Error("Error while checking text file", "error", err)
+			return box.Render("\n --- " + icon.Error + " Error get file info ---")
+		} else if !isText {
+			return box.Render("\n --- " + icon.Error + " Unsupported formats ---")
 		}
 	}
 
-	return box.Render("\n --- " + icon.Error + " Unsupported formats ---")
+	// At this point either format is not nil, or we can read the file
+	fileContent, err := readFileContent(itemPath, m.fileModel.width+20, previewLine)
+	if err != nil {
+		slog.Error("Error open file", "error", err)
+		return box.Render("\n --- " + icon.Error + " Error open file ---")
+	}
+
+	// We know the format of file, and we can apply syntax highlighting
+	if format != nil {
+		background := ""
+		if !Config.TransparentBackground {
+			background = theme.FilePanelBG
+		}
+		fileContent, err = ansichroma.HightlightString(fileContent, format.Config().Name, theme.CodeSyntaxHighlightTheme, background)
+		if err != nil {
+			slog.Error("Error render code highlight", "error", err)
+			return box.Render("\n --- " + icon.Error + " Error render code highlight ---")
+		}
+	}
+
+	if fileContent == "" {
+		return box.Render("\n --- empty ---")
+	}
+	fileContent = checkAndTruncateLineLengths(fileContent, m.fileModel.filePreview.width)
+	return box.Render(fileContent)
 }
 
-func (m model) commandLineInputBoxRender() string {
+func (m *model) commandLineInputBoxRender() string {
 	return m.commandLine.input.View()
 }
