@@ -88,6 +88,7 @@ type model struct {
 	toggleDotFile        bool
 	updatedToggleDotFile bool
 	toggleFooter         bool
+	firstLoadingComplete bool
 	filePanelFocusIndex  int
 	mainPanelHeight      int
 	fullWidth            int
@@ -198,7 +199,6 @@ type element struct {
 	name      string
 	location  string
 	directory bool
-	matchRate float64
 	metaData  [][2]string
 }
 
@@ -210,6 +210,9 @@ type sidebarModel struct {
 	directories []directory
 	renderIndex int
 	cursor      int
+	rename      textinput.Model
+	renaming    bool
+	searchBar   textinput.Model
 }
 
 type directory struct {
