@@ -2,12 +2,12 @@ package internal
 
 import tea "github.com/charmbracelet/bubbletea"
 
-func wheelMainAction(msg string, m model, cmd tea.Cmd) (model, tea.Cmd) {
+func wheelMainAction(msg string, m *model, cmd tea.Cmd) tea.Cmd {
 	switch msg {
 
 	case "wheel up":
 		if m.focusPanel == sidebarFocus {
-			m.controlSideBarListUp(true)
+			m.sidebarModel.controlListUp(true, m.mainPanelHeight)
 		} else if m.focusPanel == processBarFocus {
 			m.controlProcessbarListUp(true)
 		} else if m.focusPanel == metadataFocus {
@@ -22,7 +22,7 @@ func wheelMainAction(msg string, m model, cmd tea.Cmd) (model, tea.Cmd) {
 
 	case "wheel down":
 		if m.focusPanel == sidebarFocus {
-			m.controlSideBarListDown(true)
+			m.sidebarModel.controlListDown(true, m.mainPanelHeight)
 		} else if m.focusPanel == processBarFocus {
 			m.controlProcessbarListDown(true)
 		} else if m.focusPanel == metadataFocus {
@@ -35,5 +35,5 @@ func wheelMainAction(msg string, m model, cmd tea.Cmd) (model, tea.Cmd) {
 			}()
 		}
 	}
-	return m, cmd
+	return cmd
 }
